% -------------------------------------
% ---- Adaptive Sampling Algorithm ----
% -------------------------------------

function [SavingTx Accuracy Metric]=Case_Heuristics(datastream,toplot)

data_stream_training = datastream(1:100);
data_stream_test = datastream(101:end);

% ---- Initial training (std. dev. estimation, and threshold setup) ----
Weight=2.5; % Design Parameter
Margin=0; % Design Parameter
ThresholdLow = Weight*mean(abs(diff(data_stream_training)));
ThresholdHigh = ThresholdLow + 0;

% ---- Set-up ----
SamplingIntervals = [1 2 3 4 5]; % In number of samples
%SamplingIntervals = [6 12 18 24 30]; % In number of samples
%SamplingIntervals = [1];
NumSamplingIntervals = length(SamplingIntervals);

NumberSamples = length(data_stream_test);

% Performance
OnTrackSampling=0; % Performance metric

% Initial Values
next_sample=1; 
CurrentSamplingInterval = 1;
CurrentSampledValue = 0;
LastSampledValue = data_stream_test(1);
SampledData = 0; % Vector with the sampled data
sample_index=1; % To index effective sampled values
SamplingIntervalUsed = 1;

while next_sample <= NumberSamples

    % Get Data
    
    CurrentSampledValue = data_stream_test(next_sample);

    % Store the sampled data (value and time) 
    SampledData(sample_index)=CurrentSampledValue;
    SampledTime(sample_index)=next_sample;
    SamplingIntervalUsed(sample_index)=SamplingIntervals(CurrentSamplingInterval); 
    sample_index=sample_index+1;

    % Decide Next Interval

    Difference = abs(CurrentSampledValue-LastSampledValue);

    if(Difference <= ThresholdLow)
        OnTrackSampling = OnTrackSampling + 1;
        % Increase Sampling Interval
        %%% Policy 1
        CurrentSamplingInterval=min(NumSamplingIntervals,CurrentSamplingInterval+1);
        %%% Policy 2
        %CurrentSamplingInterval=NumSamplingIntervals;
    end
    if(Difference > ThresholdHigh)
        %%% Policy 1: Decrease Sampling Interval
        %CurrentSamplingInterval=max(1,CurrentSamplingInterval-1);
        %%% Policy 2: Reset to the minimum
        CurrentSamplingInterval=1;
    end

    %disp('------------------------------------');
    %disp('Current Sample | CurrentSampledValue | LastSampledValue | Difference | OnTrackSampling');
    %disp([next_sample CurrentSampledValue LastSampledValue Difference OnTrackSampling]);

    % Move forward

    next_sample = next_sample + SamplingIntervals(CurrentSamplingInterval);

    %disp('------------------------------------');
    %disp('next_sample | CurrentSamplingInterval');
    %disp([next_sample CurrentSamplingInterval]);

    LastSampledValue = CurrentSampledValue;

    %pause

end

if toplot==1
figure
subplot(3,1,1); plot(data_stream_test,'kx');
xlabel('Time');
ylabel('Values');
title('Real Data');
subplot(3,1,2); plot(SampledTime,SamplingIntervalUsed,'ro');
ylabel('Values');
title('Sampling Intervals');
xlabel('Time');
subplot(3,1,3); plot(SampledTime,SampledData,'ro');
xlabel('Time');
ylabel('Values');
title('Sampled Data');
end

EffNumberSamples = length(SampledData);
SavingTx = (NumberSamples-EffNumberSamples)/NumberSamples;
Accuracy = OnTrackSampling/(sample_index-1);
Metric = 1*Accuracy + 0.5*SavingTx;

% Results
disp('################# RESULTS #################');
disp('--- Thresholds ---');
disp([ThresholdLow ThresholdHigh])
disp('Successful Samplings | Av. Successful Samplings');
disp([OnTrackSampling OnTrackSampling/(sample_index-1)]);
disp('Total Number Samples | Real Number Samples | Savings (%)');
disp([NumberSamples EffNumberSamples 100*(NumberSamples-EffNumberSamples)/NumberSamples ]);
disp('Joint Metric: Accuracy + savings');
disp([1*Accuracy + 0.5*SavingTx]);

end