% -------------------------------------
% ------------- MAB Epsilon -----------
% -------------------------------------

function [SavingTx Accuracy Metric]=Case_MABThompson(datastream,toplot)

data_stream_training = datastream(1:100);
data_stream_test = datastream(101:end);

% ---- Initial training (std. dev. estimation, and threshold setup) ----
Weight=2.5; % Design Parameter
Margin=0; % Design Parameter
ThresholdLow = Weight*mean(abs(diff(data_stream_training)));
ThresholdHigh = ThresholdLow + 0;

% ---- Set-up ----
SamplingIntervals = [1 2 3 4 5]; % In number of samples
%SamplingIntervals = [6 12 18 24 30]; % In number of samples

%SamplingIntervals = [1];
NumSamplingIntervals = length(SamplingIntervals);
NumberSamples = length(data_stream_test);

% For TS
n_times_T_selected_TS = zeros(1,NumSamplingIntervals);
sigm=SamplingIntervals;
alfa=0.5;

% Performance
OnTrackSampling=0; % Performance metric
RewardActions = zeros(1,length(SamplingIntervals));

% Initial Values
next_sample=1; 
CurrentSamplingInterval = 1;
CurrentSampledValue = 0;
LastSampledValue = data_stream_test(1);
SampledData = 0; % Vector with the sampled data
sample_index=1; % To index effective sampled values
SamplingIntervalUsed = 1;
n_times_T_selected_TS(CurrentSamplingInterval)=1;

while next_sample <= NumberSamples

    % Get Data
    
    CurrentSampledValue = data_stream_test(next_sample);

    % Store the sampled data (value and time) 
    SampledData(sample_index)=CurrentSampledValue;
    SampledTime(sample_index)=next_sample;
    SamplingIntervalUsed(sample_index)=SamplingIntervals(CurrentSamplingInterval); 
    sample_index=sample_index+1;

    % Decide Next Interval
    Difference = abs(CurrentSampledValue-LastSampledValue);
    InstantaneousReward = 0;
    if(Difference <= ThresholdLow)
        OnTrackSampling = OnTrackSampling + 1;
        InstantaneousReward = SamplingIntervals(CurrentSamplingInterval);
    end

    %RewardActions(CurrentSamplingInterval) = alfa*RewardActions(CurrentSamplingInterval) + (1-alfa)*InstantaneousReward;

    RewardActions(CurrentSamplingInterval)=...
        (RewardActions(CurrentSamplingInterval).*(n_times_T_selected_TS(CurrentSamplingInterval)-1)+...
        InstantaneousReward)/(n_times_T_selected_TS(CurrentSamplingInterval));

    % TS Gaussian PDFs
    Sampling_arms=zeros(1,NumSamplingIntervals);
    for i=1:NumSamplingIntervals
        m = RewardActions(i);
        sigm(i) = (SamplingIntervals(i)/(n_times_T_selected_TS(i)+1));
        Sampling_arms(i)= m + sigm(i).*randn();
    end
    disp('Variance');
    disp(sigm);
    disp('Sampling Arms');
    disp(Sampling_arms);

    % Selecting the highest oen
    [~,b]=max(Sampling_arms);

    CurrentSamplingInterval = b;
    n_times_T_selected_TS(CurrentSamplingInterval) = n_times_T_selected_TS(CurrentSamplingInterval) + 1;


    disp('------------------------------------');
    disp('Current Sample | CurrentSampledValue | LastSampledValue | Difference | OnTrackSampling');
    disp([next_sample CurrentSampledValue LastSampledValue Difference OnTrackSampling]);
    disp('------------- Rewards --------');
    disp(RewardActions);
    disp('Num times each action is selected');
    disp(n_times_T_selected_TS);

 
    % Move forward
    next_sample = next_sample + SamplingIntervals(CurrentSamplingInterval);

%     disp('------------------------------------');
%     disp('next_sample | CurrentSamplingInterval');
%     disp([next_sample CurrentSamplingInterval]);

    LastSampledValue = CurrentSampledValue;

%   pause

end

if toplot==1
figure
subplot(3,1,1); plot(data_stream_test,'kx');
xlabel('Time');
ylabel('Values');
title('Real Data');
subplot(3,1,2); plot(SampledTime,SamplingIntervalUsed,'ro');
ylabel('Values');
title('Sampling Intervals');
xlabel('Time');
subplot(3,1,3); plot(SampledTime,SampledData,'ro');
xlabel('Time');
ylabel('Values');
title('Sampled Data');


x = [0:0.01:10]; 
y1 = normpdf(x,RewardActions(1),sigm(1));
y2 = normpdf(x,RewardActions(2),sigm(2));
y3 = normpdf(x,RewardActions(3),sigm(3));
y4 = normpdf(x,RewardActions(4),sigm(4));
y5 = normpdf(x,RewardActions(5),sigm(5));

figure
plot(x,y1./max(y1));
hold
plot(x,y2./max(y2));
plot(x,y3./max(y3));
plot(x,y4./max(y4));
plot(x,y5./max(y5));
hold
xlabel('values');
ylabel('pdf');
legend('T=1','T=2','T=3','T=4','T=5');


end

EffNumberSamples = length(SampledData);
SavingTx = (NumberSamples-EffNumberSamples)/NumberSamples;
Accuracy = OnTrackSampling/(sample_index-1);
Metric = 1*Accuracy + 0.5*SavingTx;

% Results
disp('################# RESULTS #################');
disp('--- Thresholds ---');
disp([ThresholdLow ThresholdHigh])
disp('Successful Samplings | Av. Successful Samplings');
disp([OnTrackSampling OnTrackSampling/(sample_index-1)]);
disp('Total Number Samples | Real Number Samples | Savings (%)');
disp([NumberSamples EffNumberSamples 100*(NumberSamples-EffNumberSamples)/NumberSamples ]);
disp('Joint Metric: Accuracy + savings');
disp([1*Accuracy + 0.5*SavingTx]);

end